require 'net/http'
require 'digest/sha1'
require 'json'

module UCloud
  SERVER_URI = 'http://api.spark.ucloud.cn'

  class Client
    def initialize(public_key, private_key, region)
      @public_key, @private_key, @region = public_key, private_key, region
      @uri = URI(UCloud::SERVER_URI)
    end

    def execute(action, parameters = {})
      params = parameters.inject({}) do |memo, pair|
        if pair[1].is_a?(Array)
          pair[1].each_with_index{|e, i| memo[[pair[0],i].join('.').to_sym] = e}
        else
          memo[pair[0].to_sym] = pair[1]
        end
        memo
      end
      params[:Region] ||= @region
      params.merge!(Action: action, PublicKey: @public_key)

      res = Net::HTTP.post_form(@uri, sign(params))
      JSON.parse(res.body)
    end

    private
    def sign(params)
       params.merge!(Signature: Digest::SHA1.hexdigest(params.sort.flatten.join + @private_key))
    end
  end
end
